:
# NAME:
#	config_funcs.sh - functions used by install_files et al
#
# DESCRIPTION:
#	This module is sourced by each of the scripts run by config.sh
#	(actually rc.sh).  It provides facilities like Dirname, Which
#	and various others.
#
#	It also evaluates the argument passed to the script.  It
#	currently understands:
#
#	'show'	Just show which files would need to be updated.
#
#	'debug'	As for 'show', but also shows the commands that would
#		be executed.
#
#	Both of the above set a variable DOIT which prefixes all the
#	commands issued by these scripts.  If DOIT=: then nothing
#	happens, if DOIT=echo the commands are shown but not executed.
#
#	If the argument contains an ``='' then the arguments are
#	evaled so that for example:
#.nf
#
#	config.sh ``EXT=test DOIT=:''
#.fi
#
#	Will cause install_files and friends to look for lists with an
#	extention of '.test' instead of 'list', and the commands will
#	not be executed.  Note that the quotes are needed to make
#	config.sh treat the list as a single argument.
#
# SEE ALSO:
#	install_files(8), config_setup.sh(8), remove_files(8), set_perms(8)
#
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#
#	

# RCSid:
#	$Id: config_funcs.sh,v 1.60 2023/03/18 16:52:32 sjg Exp $
#
#	@(#) Copyright (c) 1995 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

_SOURCE_SH=:

_source() {
	f=$1; shift

	for d in ${*:-`IFS=:; echo $PATH`}
	do
		test -s $d/$f && { . $d/$f; break; }
	done
}

_source /etc/rc_d/source.sh $CONFIGS

$_OS_SH _source /etc/rc_d/os.sh $CONFIGS ""
$_DEBUG_SH _source /etc/rc_d/debug.sh $CONFIGS ""

DebugOn $Myname:all

EXT=${EXT:-list}
# make sure these are set
TMP=${TMP:-/tmp}
# a separate list for each player
INCLUDED_LIST=$TMP/$Myname.included
hash=${hash:-sha1}
HASH=${HASH:-`echo $hash | toUpper`}
eval HASH_CMD=\${HASH_CMD:-\${$HASH:-$hash}}
HASH_LIST=${HASH_LIST:-$TMP/CONFIG_HASH_LIST}
CHANGE_LIST=${CHANGE_LIST:-$TMP/CONFIG_CHANGES}
REBOOT_LIST=${REBOOT_LIST:-$TMP/REBOOT}
INSTALL_LIST=${INSTALL_LIST:-$TMP/install_files.list.$HOST}

# pick up Which(), Dirname() and exists()
$_FUNCS_SH _source /etc/rc_d/funcs.sh $Mydir/.. $CONFIGS ""
_source /etc/rc_d/twdl.sh $Mydir/.. $CONFIGS ""
_source /etc/rc_d/install_new.sh $Mydir/.. $CONFIGS ""
_source /etc/rc_d/finfo.sh $Mydir/.. $CONFIGS ""
TWDL=${TWDL:-:}

# We really won't handle it if -h doesn't work..
test_L=${test_L:--h}

_source config_listfuncs.sh  $Mydir $CONFIGS/config.d

tok() {
	(t=$1; test "$3" && IFS="$3"; set -- $2; eval echo '$'$t )
}

# because we eval .if lines in one go 
# and apply ! to the total result, 
# the following allows use of missing instead of ! exists so that
#     .if missing file && exists other_file
# will work as expected

missing() {
	exists $*
	[ $? != 0 ]
}

file_size() {
	set -- `$DU $1`
	echo $1
}

file_cksum() {
	file=$1
	list=$2
	
	if [ "$list" -a -s ${list:-/dev/null} ]; then
		set -- `grep "($file)" $list`
		case "$1$2" in
		$HASH\(*) ;;
		*)
			set -- `${HASH_CMD} $file`
			;;
		esac
	else
		set -- `${HASH_CMD} $file`
	fi
	eval echo '$'$#
}

get_uid() {
	grep "^$1:" $PATH_ETC_PASSWD | cut -d: -f3
}

get_gid() {
	grep "^$1:" $PATH_ETC_GROUP | cut -d: -f3
}

get_ugid() {
	grep "^$1:" $PATH_ETC_PASSWD | cut -d: -f4
}

get_nis() {
	${YPMATCH:-ypmatch} "$@" 2> /dev/null
}

get_nis_gid() {
	get_nis $1 group | cut -d: -f3
}

set_perms() {
	eval path=$1
	eval mode=$2
	eval owner=$3
	eval group=$4
	eval type=$5

	exists $path || return 0

	if [ "$SHARE_ARCH" != "$DEST_SHARE_ARCH" ]; then
		# it gets really messy if the DEST_OS is not the same
		# as ours
		case "$owner" in
		[0-9]*) ;;
		*)
			o=`get_uid $owner`
			[ "$o" ] && owner=$o
			;;
		esac
		case "$group" in
		[0-9]*) ;;
		*)
			g=`get_gid $group`
			$SKIP_NIS g=${g:-`get_nis_gid $group`}
			[ "$g" ] && group=$g
			;;
		esac
	fi
	#
	# this is ok for NetBSD, SunOS, Solaris
	# and HP-UX as it has both bin and sys.
	#
	case "$group" in
	root|wheel) group=${g_wheel:-0};;
	sys)	group=${g_sys:-3};;
	bin)	group=${g_bin:-3};;
	*)	eval "group=\${g_$group:-$group}";;
	esac
	
	#
	# must do chmod last, as many systems
	# reset s-[ug]id bits when owner/group are changed
	#
	$DOIT ${CHOWN:-chown} $CHOWN_h $owner $path
	$DOIT ${CHGRP:-chgrp} $CHGRP_h $group $path
	#
	# do not chmod(1) symlinks.
	#
	if [ ! $test_L $path ]; then
		$DOIT chmod $mode $path

		#
		# sgid bit only gets set via g+s
		#
		case $mode in
		*[762]???)
			$DOIT chmod g+s $path
			;;
		esac
	fi
}

space_for() {
	dir=$1
	want=$2			# kbytes or filename
	[ -f $want ] && want=`file_size $2`

	avail=0

	if [ -d $dir ]; then
		avail=`$DF $dir | awk '/%.*\// { F=NF - 2; print $F }'`
	else
		echo "$dir: does not exist" >&2
	fi
	[ ${avail:-0} -ge $want ]
}

case "$1" in
*=*)
	eval "$@"
	;;
show)
	# don't actually do anything
	DOIT=:
	export DOIT
	;;
check|debug)
	DOIT="echo + "
	export DOIT
	[ "$1" = debug ] && set -x
	;;
esac
